// ======================================================================
//
// MaterialData_3dfx.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef MATERIAL_DATA_3DFX_H
#define MATERIAL_DATA_3DFX_H

// ======================================================================

#include <glide.h>

#include "Material.h"
#include "MaterialData.h"

class Palette;
class StaticMaterial;
class PaletteData_3dfx;
struct uv_instance;

// ======================================================================

class MaterialData_3dfx : public MaterialData
{
	friend class GL_3dfx;

private:

	static u_int globalTagLRU;

private:

	const StaticMaterial   *staticMaterial;
	const PaletteData_3dfx *palette;

	int           color;
	mutable u_int tagLRU;

	mutable FxU32 ptr;
	int           size;
	GrTexInfo     info;

	bool          translucency;
	mutable bool  loaded;

	void (MaterialData_3dfx::*setupFunction)(void) const;

private:
	
	void init(void);

	void download(void) const;

	void setupNOP(void) const;
	void setupChromakeyOffColorConstantAlphaOff(void) const;
	void setupChromakeyOffColorConstantAlphaConstant(void) const;
	void setupChromakeyOffColorConstantAlphaMap(void) const;
	void setupChromakeyOffColorMapAlphaOff(void) const;
	void setupChromakeyOffColorMapAlphaMap(void) const;
	void setupChromakeyOffColorMapAlphaConstant(void) const;
	void setupChromakeyOnColorMapAlphaOff(void) const;

public:

	// this is not for you, it's for the movie player
	MaterialData_3dfx(const uv_instance *uv);

	// this is for general use
	MaterialData_3dfx(const StaticMaterial *material);
	virtual ~MaterialData_3dfx(void);

	u_int      getTagLRU(void) const;
	bool       hasTranslucency(void) const;

	void       setup(void) const;

	void       download(const uv_instance *uv, u_int32 squareXOffset, u_int32 squareYOffset) const;
	void       unload(void) const;
};

// ======================================================================

inline void MaterialData_3dfx::setup(void) const
{
	(this->*setupFunction)();
}

// ----------------------------------------------------------------------

inline u_int MaterialData_3dfx::getTagLRU(void) const
{
	return tagLRU;
}

// ----------------------------------------------------------------------

inline bool MaterialData_3dfx::hasTranslucency(void) const
{
	return translucency;
}

// ======================================================================

#endif
